
local addon = KiwiPlates

local next = next 
local pairs = pairs
local unpack = unpack
local UnitIsUnit = UnitIsUnit
local UnitIsFriend = UnitIsFriend
local UnitThreatSituation = UnitThreatSituation
local C_Timer_After = C_Timer.After
local C_GetNamePlateForUnit = C_NamePlate.GetNamePlateForUnit

-- not tanked / offtanked / insecure tanking / secure tanking
addon.defaults.threat = {
	colorsTank  = { [0] = { 1,0,0 }, [1] = { 0,0,1 }, [2] = { 1,1,0 }, [3] = { 0,1,0 }, },
	colorsOther = {	[0] = { .85,1,0 }, [1] = { .35,1,0 }, [2] = { 1,.4,0 }, [3] = { 1,0,0 }, },
}

local inCombat
local threatColors
local offtanks = {}
local group_units = { [true] = {}, [false] = { 'player', 'party1', 'party2', 'party3', 'party4' } }
for i=1,MAX_RAID_MEMBERS do table.insert( group_units[true], 'raid'..i ) end
local unitTarget  = setmetatable({}, {__index = function(t,k) local v=k.."target" t[k]=v return v end})

local CUF_UpdateHealthColor_Orig = CompactUnitFrame_UpdateHealthColor

local function UpdatePlateThreatColor(UnitFrame)
	local unit = UnitFrame.unit
	if UnitIsEnemy(unit, "player") then
		local threat = UnitThreatSituation( 'player', unit ) or 0  -- 2&3 = tanked
		if threat<2 then
			threat = 0 -- not tanked
			local target = unitTarget[unit]
			for _,utank in next, offtanks do
				if (UnitThreatSituation( utank, unit ) or 0)>=2 or UnitIsUnit(utank, target) then
					threat = 1 -- offtanked
					break
				end
			end
			if threat == 0 and UnitIsUnit( 'player', target ) then
				threat = 3
			end
		end
		if threat ~= UnitFrame.__threat then
			local healthBar = UnitFrame.healthBar			
			local r, g, b = unpack( threatColors[threat] )
			healthBar:SetStatusBarColor( r, g, b )
			-- UnitFrame.selectionHighlight:SetVertexColor( r, g, b )
			healthBar.r, healthBar.g, healthBar.b = r, g, b
			UnitFrame.__threat = threat
		end	
		return true
	end	
end

local function CUF_UpdateHealthColor(UnitFrame)
	if not ( inCombat and UnitFrame.ClassificationFrame and UpdatePlateThreatColor(UnitFrame) ) then
		CUF_UpdateHealthColor_Orig(UnitFrame)
	end	
end

local function ThreatTimer()
	if inCombat then
		for _, plateFrame in pairs(C_NamePlate.GetNamePlates()) do
			UpdatePlateThreatColor( plateFrame.UnitFrame )
		end	
		C_Timer_After(0.25, ThreatTimer)
	end
end

local function CombatStart()
	local spec = GetSpecializationRole(GetSpecialization() or 1)
	if addon.db.threat.alwaysEnabled or spec == 'TANK' then
		inCombat = true
		threatColors = spec=='TANK' and addon.db.threat.colorsTank or addon.db.threat.colorsOther
		C_Timer_After(0.25, ThreatTimer)
	end
end

local function CombatEnd()
	inCombat = false
	for _, plateFrame in pairs(C_NamePlate.GetNamePlates()) do
		local UnitFrame = plateFrame.UnitFrame
		CUF_UpdateHealthColor_Orig(UnitFrame)
		UnitFrame.__threat = nil
	end
end

local function UpdateTanks()
	wipe(offtanks)
	local m = GetNumGroupMembers()
	if m>1 then
		local units = group_units[IsInRaid()]
		for i=1,m do
			local unit = units[i]
			if UnitGroupRolesAssigned(unit) == 'TANK' and (not UnitIsUnit(unit,'player')) then
				offtanks[#offtanks+1] = unit
			end
		end
	end
end

--===============================================================

local threat = addon:NewModule('threat')

addon.GROUP_ROSTER_UPDATE   = UpdateTanks
addon.PARTY_MEMBERS_CHANGED = UpdateTanks
addon.ARENA_OPPONENT_UPDATE = UpdateTanks
addon.PLAYER_REGEN_DISABLED = CombatStart
addon.PLAYER_REGEN_ENABLED  = CombatEnd

function threat:Update()
	if addon.db.threat.enabled then 
		CompactUnitFrame_UpdateHealthColor = CUF_UpdateHealthColor
		addon:RegisterEvent('GROUP_ROSTER_UPDATE')
		addon:RegisterEvent('PARTY_MEMBERS_CHANGED')
		addon:RegisterEvent('ARENA_OPPONENT_UPDATE')
		addon:RegisterEvent('PLAYER_REGEN_DISABLED')
		addon:RegisterEvent('PLAYER_REGEN_ENABLED')
		UpdateTanks()
	else
		CompactUnitFrame_UpdateHealthColor = CUF_UpdateHealthColor_Orig
		addon:UnregisterEvent('GROUP_ROSTER_UPDATE')
		addon:UnregisterEvent('PARTY_MEMBERS_CHANGED')
		addon:UnregisterEvent('ARENA_OPPONENT_UPDATE')
		addon:UnregisterEvent('PLAYER_REGEN_DISABLED')
		addon:UnregisterEvent('PLAYER_REGEN_ENABLED')
		inCombat = false
	end	
end

--===============================================================

addon:SetupOptions( 'Skins/Settings', 'Threat', {
	enabled = {
		type = "toggle",
		order = 1,
		name = "Colorize Health Bar",
		get = function() return addon.db.threat.enabled end,
		set = function ( _, value) 
			addon.db.threat.enabled = value or nil
			addon:Update(true)
		end,
	},
	separator1 = {
		type = "header",
		order = 9,
		name = "Colors for Tanks",
	},
	colorTank0 = {
		type = "color",
		order = 10,
		hasAlpha = false,		
		name = "Not Tanking",
		get = function() 
			return unpack( addon.db.threat.colorsTank[0] ) 
		end,
		set = function( _, r,g,b ) 
			addon.db.threat.colorsTank[0] = { r, g, b }
		end, 
		disabled = function() return not addon.db.threat.enabled end,
	},
	colorTank1 = {
		type = "color",
		order = 11,
		hasAlpha = false,		
		name = "Offtanked",
		desc = "Tanked by another tank",
		get = function() 
			return unpack( addon.db.threat.colorsTank[1] ) 
		end,
		set = function( _, r,g,b ) 
			addon.db.threat.colorsTank[1] = { r, g, b }
		end, 
		disabled = function() return not addon.db.threat.enabled end,
	},
	colorTank2 = {
		type = "color",
		order = 12,
		hasAlpha = false,		
		name = "Insecure Tanking",
		get = function() 
			return unpack( addon.db.threat.colorsTank[2] ) 
		end,
		set = function( _, r,g,b ) 
			addon.db.threat.colorsTank[2] = { r, g, b }
		end, 
		disabled = function() return not addon.db.threat.enabled end,
	},
	colorTank3 = {
		type = "color",
		order = 13,
		hasAlpha = false,		
		name = "Secure Tanking",
		get = function() 
			return unpack( addon.db.threat.colorsTank[3] ) 
		end,
		set = function( _, r,g,b ) 
			addon.db.threat.colorsTank[3] = { r, g, b }
		end, 
		disabled = function() return not addon.db.threat.enabled end,
	},
	separator2 = {
		type = "header",
		order = 19,
		name = "Colors for DPS&Healers",
		disabled = function() return not addon.db.threat.enabled end,
		hidden = function() return not addon.db.threat.alwaysEnabled end,		
	},
	colorOther0 = {
		type = "color",
		order = 20,
		hasAlpha = false,		
		name = "Not Tanked",
		desc = "Mob attacking another healer or dps.",
		get = function() 
			return unpack( addon.db.threat.colorsOther[0] ) 
		end,
		set = function( _, r,g,b ) 
			addon.db.threat.colorsOther[0] = { r, g, b }
		end, 
		disabled = function() return not addon.db.threat.enabled end,
		hidden = function() return not addon.db.threat.alwaysEnabled end,		
	},
	colorOther1 = {
		type = "color",
		order = 21,
		hasAlpha = false,		
		name = "Tanked",
		desc = "Tanked by some tank",
		get = function() 
			return unpack( addon.db.threat.colorsOther[1] ) 
		end,
		set = function( _, r,g,b ) 
			addon.db.threat.colorsOther[1] = { r, g, b }
		end, 
		disabled = function() return not addon.db.threat.enabled end,
		hidden = function() return not addon.db.threat.alwaysEnabled end,		
	},
	colorOther2 = {
		type = "color",
		order = 22,
		hasAlpha = false,		
		name = "Attacking me",
		desc = "Attacking me (low aggro)",
		get = function() 
			return unpack( addon.db.threat.colorsOther[2] ) 
		end,
		set = function( _, r,g,b ) 
			addon.db.threat.colorsOther[2] = { r, g, b }
		end,
		disabled = function() return not addon.db.threat.enabled end,
		hidden = function() return not addon.db.threat.alwaysEnabled end,		
	},
	colorOther3 = {
		type = "color",
		order = 23,
		hasAlpha = false,		
		name = "Attacking me",
		desc = "Attacking me (high aggro)",
		get = function() 
			return unpack( addon.db.threat.colorsOther[3] ) 
		end,
		set = function( _, r,g,b ) 
			addon.db.threat.colorsOther[3] = { r, g, b }
		end,
		disabled = function() return not addon.db.threat.enabled end,
		hidden = function() return not addon.db.threat.alwaysEnabled end,		
	},
	separator3 = {
		type = "header",
		order = 30,
		name = "Special",
	},
	alwaysEnabled = {
		type = "toggle",
		order = 31,
		width = "full",
		name = "Apply threat colors only to Tank specs",
		get = function() return not addon.db.threat.alwaysEnabled end,
		set = function ( _, value) 
			addon.db.threat.alwaysEnabled = (not value) or nil
		end,
		disabled = function() return not addon.db.threat.enabled end,		
	},
} )
